# -*- coding: utf-8 -*-
#
# Python demo program for executing some initial tests of 'RemoSens'
# 
# 'RemoSens' is a LAN/WiFi interface-box for legacy
# R&S USB power sensors
# 
# This programming example tries to execute both 'Continuous Average'
# and 'Power Trace' measurements.
#   HINT: If a R&S power sensor is used which does not provide
#         a 'Trace' function (like the thermal power sensors
#         of the NRP-Z5x series), the Trace measurements will fail!
# 
# Communication is preferably done via TCP/IP VISA HiSLIP connection
# (although also SCPI-RAW communication is also supported by 'RemoSens')
# ___________________________________________________________________________
#
# To use this demo program, you need to have install the following:
#
#  Python 3.x        https://python.org/download/
#
#  PyVISA >= 1.8     https://github.com/pyvisa/pyvisa
#
#
#  +-- MEMORY AID -------------------------------------------------------------------------------------------+
#  |                                                                                                         |
#  |  How to install or upgrade a Python module XXXXX                                                        |
#  |                                                                                                         |
#  |  --> In case of a direct Internet connection:                                                           |
#  |                                                                                                         |
#  |         python -m pip install --upgrade XXXXX                                                           |
#  |                                                                                                         |
#  |      Example:                                                                                           |
#  |         python -m pip install --upgrade pyvisa                                                          |
#  |                                                                                                         |
#  |                                                                                                         |
#  |  --> In case you are behind a PROXY                                                                     |
#  |         (replace 'user', 'password', 'proxy-host.example.com' and 'port' with your locally valid data): |
#  |                                                                                                         |
#  |         set http_proxy=http://user:password@proxy-host.example.com:port                                 |
#  |         set https_proxy=http://user:password@proxy-host.example.com:port                                |
#  |         python -m pip install --upgrade XXXXX                                                           |
#  |                                                                                                         |
#  +---------------------------------------------------------------------------------------------------------+
#
# ____________________________________________________________________________
#
#  Author            Juergen D. Geltinger
#                    IFD - Ing.-Buero fuer Datentechnik
#                    Eugenbacher Str. 93a
#                    DE-84032 Altdorf
#                    Germany
#
#  Support           support@ifd.de
#
#  History           2022-02-18   JDG  Added optional backend-parameter;
#                                      added more comments/hints
#                    2022-02-17   JDG  Added SCPI-RAW communication possibility
#                    2022-01-12   JDG  Added zero calibration (optional)
#                    2020-09-24   JDG  Added save/restore settings
#                    2020-08-15   JDG  Added Trace measurement
#                    2020-06-10   JDG  Added more comments
#                    2020-05-20   JDG  Initial release
#
# ____________________________________________________________________________

import os
import sys
import socket
import pyvisa as visa
from time import sleep
from math import fabs, log10
from configparser import ConfigParser

# Importing the 'RemoteSensor' class (implemented in module 'RemoSens.py')
# We first try to find the module locally. If it does not exist locally,
# we expect it in a folder, in parallel to the current one, with
# folder-name 'RemoSens' -- in other words, we expect to find it as
# ../RemoSens/RemoSens.py
try:
    import RemoSens
except ModuleNotFoundError:
    sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'RemoSens')))
import RemoSens

from RemoSens import RemoteSensor


# We will execute 5 measurements (in 'ContAV' and 'Trace' mode)
NUM_MEAS = 5

                
#-----------------------------------------------------
# Configure a Continuous Average Measurement example
#-----------------------------------------------------
def configure_cont_av( device ):
    device.sendCmd( "*rst"                    )   # Reset
    device.sendCmd( "*cls"                    )   # Clear status queue
    device.sendCmd( "sens:func \"power:avg\"" )   # Power-Average mode
    device.sendCmd( "sens:freq 1.0e9"         )   # Setting measurement frequency
    device.sendCmd( "sens:aver:coun:auto off" )   # Auto-Averaging OFF
    device.sendCmd( "sens:aver:count 16"      )   # Setting depth of Avg Filter
    device.sendCmd( "sens:aver:stat on"       )   # Setting Avg Filter ON
    device.sendCmd( "trig:sour imm"           )   # Immediate trigger
    device.sendCmd( "sens:pow:avg:aper 1e-5"  )   # Aperture time
    device.sendCmd( "unit:power w"            )   # We expect Watt results

    # Any errors in our configuration?
    device.query_syst_err(verbose=True)


#-----------------------------------------------------
# Configure a Trace Measurement example
#-----------------------------------------------------
def configure_trace( device ):
    device.sendCmd( "*rst"                    )   # Reset
    device.sendCmd( "*cls"                    )   # Clear status queue
    device.sendCmd( "sens:func \"xtim:pow\""  )   # Trace mode
    device.sendCmd( "sens:freq 1.0e9"         )   # Setting measurement frequency
    device.sendCmd( "sens:trac:aver:coun 4"   )   # Setting depth of Trace Avg Filter
    device.sendCmd( "sens:trac:aver:stat on"  )   # Activate Trace Avg Filter
    device.sendCmd( "trig:sour imm"           )   # Immediate trigger
    device.sendCmd( "sens:trac:time 10e-3"    )   # Length of Trace = 10 ms
    device.sendCmd( "sens:trac:poin 2000"     )   # 2000 trace points --> Take care to read enough data in a FETCH?
                                                 #                       In ASCII output format each trace point comes
                                                 #                       as an exponential number like 1.234567e-09 and
                                                 #                       the points are separated by commas ','
                                                 #                       Hence, for 2000 points you should read into an
                                                 #                       input buffer with a capacity of at least
                                                 #                       2000 * (13 + 1)  = 28000 bytes
                                                 #                       See the size in receive_data() function
    device.sendCmd( "unit:power w"            )   # We expect Watt results

    # Any errors in our configuration?
    device.query_syst_err(verbose=True)


#-----------------------------------------------------
# This function executes a given number of single
# ContAv measurements and prints the results
#-----------------------------------------------------
def execute_cont_av_measurements( device, num_meas ):

    for n in range(num_meas):

        # Start a new measurement
        device.sendCmd( "INIT:IMM" )

        bMeasReady = False

        # Preventing a "deadlock" while waiting...
        for i in range(80):
            uiSTB = device.readSTB()

            if (uiSTB & RemoSens.MessAvailBit) != 0:
                bMeasReady = True
                break

            sleep( 0.250 )

        if not bMeasReady:
            return False
            
        device.sendCmd( "FETCH?" )
        data = device.read()
 
        if not data:
            return False
        
        avg, auxmin, auxmax = data.rstrip().split(',') 
        
        print("Result of measurement #{:03d}: {:>13} W; {:6.2f} dBm".format(n + 1, avg, device.watt_to_dbm(float(avg))))

    return True


#-----------------------------------------------------
# This function executes a given number of
# Trace measurements and prints the results
#-----------------------------------------------------
def execute_trace_measurements( device, num_meas ):

    for n in range(num_meas):

        # Start a new measurement
        device.sendCmd( "INIT:IMM" )

        bMeasReady = False

        # Preventing a "deadlock" while waiting...
        for i in range(80):
            uiSTB = device.readSTB()

            if (uiSTB & RemoSens.MessAvailBit) != 0:
                bMeasReady = True
                break

            sleep( 0.250 )

        if not bMeasReady:
            return False
            
        device.sendCmd( "FETCH?" )
        data = device.read()
 
        if not data:
            return False
        
        results = data.rstrip().split(',') 
        
        print("Result of measurement #{:03d}:".format(n + 1))
        print(results)

        import matplotlib.pyplot as plt
        plt.plot(list(float(x) for x in results))
        plt.show()

    return True



if __name__ == "__main__":
    # Get a possibly already defined host or IP from a local INI file
    # ---------------------------------------------------------------
    config = ConfigParser()
    config.read('InitialTest.ini')
    strCurrentHost = config.get('Main', 'HostOrIP', fallback = '192.168.1.200')
    # ---------------------------------------------------------------


    if not strCurrentHost:
        strPrompt = "Enter Hostname or IP address [<none>]: "
    else:
        strPrompt = "Enter Hostname or IP address [" + strCurrentHost + "]: "
    devNameOrIP = input( strPrompt );
    if not devNameOrIP and strCurrentHost:
        devNameOrIP = strCurrentHost

    if not devNameOrIP and len(sys.argv) > 1:
      devNameOrIP = sys.argv[1]
    
    if not devNameOrIP:
        print( "Error: Device Name or IP address missing" );
        print( "  Please give the device's hostname or IP as a parameter" );
        print( "  Example:" );
        print( "    %s  NRP33SN-900006" % sys.argv[0] );
        sys.exit(-1)

    # Save the current hostname or IP address for a possibly later use
    # ----------------------------------------------------------------
    if not config.has_section('Main'):
        config.add_section('Main')
    config.set('Main', 'HostOrIP', devNameOrIP)
    with open('InitialTest.ini', 'w') as f:
        config.write(f)
    # ----------------------------------------------------------------


    # Defining a VISA resource, which is to be communicated with...
    # ----------------------------------------------------------------
    #
    # This can be either be a VISA HiSLIP communication (= recommended)
    # or a SCPI-RAW communication (also known as SOCKET 5025)
    #
    # ----------------------------------------------------------------
    # Either use HiSLIP resource (recommended)...
    remoteSensorResource = 'TCPIP::{}::HISLIP'.format(devNameOrIP)
    # ...or SCPI-RAW resource
    #remoteSensorResource = 'TCPIP::{}::5025::SOCKET'.format(devNameOrIP)

    # --------------------------------------------------------------------
    # Instantiating our RemoSens device (by the resource as defined above)
    #
    # Hint: optionally a certain 'backend' parameter can be specified.
    #       For example, if 'PyVISA-py' is to be used as a backend VISA
    #       implementation, call
    #
    #         sensor = RemoteSensor(remoteSensorResource, '@py')
    #
    #       Keep in mind, that 'PyVISA-py' does not have a HiSLIP
    #       implementation. Thus, only SCPI-RAW communication is
    #       possible with 'RemoSens' devices in this case!
    # --------------------------------------------------------------------
    sensor = RemoteSensor(remoteSensorResource)

    # Information about Interface-Box
    print("'RemoSens' Interface-Box Version: " + sensor.query("SYST:IFVersion?"))
    
    # Ask which R&S power sensor is connected
    print(sensor.identify())

    # If a Zero Calibration of the power sensor is desired, un-comment the following
    sensor.zero_calibration(verbose=True)

    # Executing some Continuous Average measurements 
    configure_cont_av(sensor)
    execute_cont_av_measurements(sensor, NUM_MEAS)

    # Executing some Trace measurements
    # (HINT: Not all Power Sensors are capable of doing Trace measurement!) 
    configure_trace(sensor)
    execute_trace_measurements(sensor, NUM_MEAS)

    # Close and finish program
    sensor.close()
    
