# -*- coding: utf-8 -*-
#
# Abstract   Python examples for Remote Sensor Interface 'RemoSens'
#
# Purpose    Utility and Device I/O functions
#
#____________________________________________________________________________
#
# Author     Juergen D. Geltinger
#            IFD - Ing.-Buero fuer Datentechnik
#            Eugenbacher Str. 93a
#            DE-84032 Altdorf
#            Germany
#
#  Support   support@ifd.de
#
# History    2022-02-18   JDG  Added optional backend-parameter
#            2020-05-20   JDG  Initial release
#
# ____________________________________________________________________________

import sys
import socket
import pyvisa as visa
from math import fabs, log10
from time import sleep


# Definition of some RemoSenso Status-Bit values
ZeroCalBit    = 1<<0
DevStatusBit  = 1<<1
ErrQueueBit   = 1<<2
StatQuestBit  = 1<<3
MessAvailBit  = 1<<4
EvtStatRegBit = 1<<5
ServiceReqBit = 1<<6
StatOperBit   = 1<<7


#====================================================
# Defining a class which represents a network sensor
# Includes some utility functions
#====================================================
class RemoteSensor:    
    def __init__(self, res, backend=''):
        self.resource = res
        self.rm = visa.ResourceManager(backend)
        self.remosens = self.rm.open_resource(self.resource)
        
        # Very important for SCPI-RAW communication
        self.remosens.read_termination = '\n'
        self.remosens.write_termination = '\n'
        self.remosens.timeout = 20000

    def watt_to_dbm( self, watt ):
        if fabs(watt) < 1.0e-20:
            return -170.0

        return 10.0 * log10( fabs(watt) ) + 30.0

    def close( self ):
        self.remosens.close()


    #-----------------------------------------------------
    # Set communication timeout (ms)
    #-----------------------------------------------------
    def setTimeout( self, timo ):
        self.remosens.timeout = timo


    #-----------------------------------------------------
    # Sends a command
    #-----------------------------------------------------
    def sendCmd( self, cmd ):
        # Make sure that exactly ONE LineFeed is in place
        cmdToSend = cmd
        if self.remosens.write_termination == '\n':
            while cmdToSend.endswith('\n'):
                cmdToSend = cmdToSend[:-1]
        else:
            if not cmdToSend.endswith('\n'):
                cmdToSend = cmdToSend + '\n'

        self.remosens.write( cmdToSend )


    #-----------------------------------------------------
    # Reads data from a device
    #-----------------------------------------------------
    def read( self ):
        return self.remosens.read()


    #-----------------------------------------------------
    # Send query command to the device and return answer
    #-----------------------------------------------------
    def query( self, cmd ):
        cmdQuery = cmd
        if self.remosens.write_termination == '\n':
            while cmdQuery.endswith('\n'):
                cmdQuery = cmdQuery[:-1]
        else:
            if not cmdQuery.endswith('\n'):
                cmdQuery = cmdQuery + '\n'
        return self.remosens.query(cmdQuery)


    #-----------------------------------------------------
    # Read the status byte
    #-----------------------------------------------------
    def readSTB( self ):
        return int(self.query( "*STB?" ))


    #-----------------------------------------------------
    # Send *IDN? command and read response
    #-----------------------------------------------------
    def identify( self ):
        return self.query( "*IDN?" )


    #-----------------------------------------------------
    # Query System Error
    #-----------------------------------------------------
    def query_syst_err( self, verbose=False ):
        systErr = self.query( "SYST:ERR?" )

        if verbose:
            print("SYST:ERR? returned %s" % systErr )
            
        return systErr

    #-----------------------------------------------------
    # Zero Calibration
    #-----------------------------------------------------
    def zero_calibration( self, verbose=False ):
        if verbose:
            print("Switch OFF any RF input power to the sensor for Zero Calibration. Then press ENTER...")
            input()

        self.sendCmd( "cal:zero:auto once" )   # Start Zero Calibration

        for i in range(50):
            calStat = self.query("STAT:OPER:COND?")
            if int(calStat) & ZeroCalBit:
              if verbose:
                  print("Zero Calibration in progress...")
            else:
              break
            sleep( 0.250 )

        if verbose:
            print("Switch ON RF input power to the sensor. Then press ENTER...")
            input()

#=====================================================
# End of 'RemoteSensor' class definition 
#====================================================
